from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class AchievementsMigration(MigrationABC):
    name = "1.1.0_AchievementsMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `Achievements` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `ServerId` BIGINT,
                        `Name` VARCHAR(255) NOT NULL,
                        `Attribute` VARCHAR(255) NOT NULL,
                        `Operator` VARCHAR(2) NOT NULL,
                        `Value` VARCHAR(255) NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`),
                        FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `AchievementsHistory`
                    (
                        `Id`    BIGINT(20)  NOT NULL,
                        `ServerId` BIGINT,
                        `Name` VARCHAR(255) NOT NULL,
                        `Attribute` VARCHAR(255) NOT NULL,
                        `Operator` VARCHAR(2) NOT NULL,
                        `Value` VARCHAR(255) NOT NULL,
                        `Deleted`   BOOL                 DEFAULT FALSE,
                        `DateFrom`  DATETIME(6) NOT NULL,
                        `DateTo`    DATETIME(6) NOT NULL
                    );
                """
            )
        )

        self._cursor.execute(str(f"""ALTER TABLE Users ADD MessageCount BIGINT NOT NULL DEFAULT 0 AFTER XP;"""))
        self._cursor.execute(str(f"""ALTER TABLE Users ADD ReactionCount BIGINT NOT NULL DEFAULT 0 AFTER XP;"""))
        self._cursor.execute(str(f"""ALTER TABLE UsersHistory ADD MessageCount BIGINT NOT NULL DEFAULT 0 AFTER XP;"""))
        self._cursor.execute(str(f"""ALTER TABLE UsersHistory ADD ReactionCount BIGINT NOT NULL DEFAULT 0 AFTER XP;"""))

        self._cursor.execute(
            str(
                f"""
                    DROP TRIGGER IF EXISTS `TR_AchievementsUpdate`;

                    CREATE TRIGGER `TR_AchievementsUpdate`
                        AFTER UPDATE
                        ON `Achievements`
                        FOR EACH ROW
                    BEGIN
                        INSERT INTO `AchievementsHistory` (
                            `Id`, `Name`, `Attribute`, `Operator`, `Value`, `ServerId`, `DateFrom`, `DateTo`
                        )
                        VALUES (
                            OLD.Id, OLD.Name, OLD.Attribute, OLD.Operator, OLD.Value, OLD.ServerId, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6)
                        );
                    END;
                    
                    DROP TRIGGER IF EXISTS `TR_AchievementsDelete`;
                    
                    CREATE TRIGGER `TR_AchievementsDelete`
                        AFTER DELETE
                        ON `Achievements`
                        FOR EACH ROW
                    BEGIN
                        INSERT INTO `AchievementsHistory` (
                            `Id`, `Name`, `Attribute`, `Operator`, `Value`, `ServerId`, `Deleted`, `DateFrom`, `DateTo`
                        )
                        VALUES (
                            OLD.Id, OLD.Name, OLD.Attribute, OLD.Operator, OLD.Value, OLD.ServerId, TRUE, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6)
                        );
                    END;
                """
            ),
            multi=True,
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `Achievements`;")

        self._cursor.execute(str(f"""ALTER TABLE Users DROP COLUMN MessageCount;"""))
        self._cursor.execute(str(f"""ALTER TABLE Users DROP COLUMN ReactionCount;"""))
