from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.achievement import Achievement


class AchievementRepositoryService(AchievementRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        AchievementRepositoryABC.__init__(self)

    def _from_result(self, result: tuple):
        return Achievement(
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[2]),
            result[6],
            result[7],
            id=result[0],
        )

    def get_achievements(self) -> List[Achievement]:
        achievements = List(Achievement)
        self._logger.trace(__name__, f"Send SQL command: {Achievement.get_select_all_string()}")
        results = self._context.select(Achievement.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user with id {result[0]}")
            achievements.append(self._from_result(result))

        return achievements

    def get_achievement_by_id(self, id: int) -> Achievement:
        self._logger.trace(__name__, f"Send SQL command: {Achievement.get_select_by_id_string(id)}")
        result = self._context.select(Achievement.get_select_by_id_string(id))[0]

        return self._from_result(result)

    def get_achievements_by_server_id(self, server_id: int) -> List[Achievement]:
        achievements = List(Achievement)
        self._logger.trace(__name__, f"Send SQL command: {Achievement.get_select_by_id_string(server_id)}")
        results = self._context.select(Achievement.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user with id {result[0]}")
            achievements.append(self._from_result(result))

        return achievements

    def add_achievement(self, achievement: Achievement):
        self._logger.trace(__name__, f"Send SQL command: {achievement.insert_string}")
        self._context.cursor.execute(achievement.insert_string)

    def update_achievement(self, achievement: Achievement):
        self._logger.trace(__name__, f"Send SQL command: {achievement.udpate_string}")
        self._context.cursor.execute(achievement.udpate_string)

    def delete_achievement(self, achievement: Achievement):
        self._logger.trace(__name__, f"Send SQL command: {achievement.delete_string}")
        self._context.cursor.execute(achievement.delete_string)
