from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from modules.permission.service.permission_service import PermissionService


class AchievementMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        achievements: AchievementRepositoryABC,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        permissions: PermissionService,
    ):
        QueryABC.__init__(self, "AchievementMutation")

        self._servers = servers
        self._achievements = achievements
        self._bot = bot
        self._db = db
        self._permissions = permissions

        self.set_field("updateAchievement", self.resolve_update_achievement)

    def resolve_update_achievement(self, *_, input: dict):
        achievement = self._achievements.get_achievement_by_id(input["id"])
        self._can_user_mutate_data(achievement.server, UserRoleEnum.moderator)

        achievement.name = input["name"] if "name" in input else achievement.name
        achievement.attribute = input["attribute"] if "attribute" in input else achievement.attribute
        achievement.operator = input["operator"] if "operator" in input else achievement.operator
        achievement.value = input["value"] if "value" in input else achievement.value

        self._achievements.update_achievement(achievement)
        self._db.save_changes()

        achievement = self._achievements.get_achievement_by_id(input["id"])
        return achievement
