import discord
from cpl_core.logging import LoggerABC
from cpl_discord.events.on_scheduled_event_update_abc import OnScheduledEventUpdateABC
from cpl_discord.service import DiscordBotServiceABC
from discord import EventStatus

from modules.base.model.active_event import ActiveEvent
from modules.base.service.event_service import EventService


class BaseOnScheduledEventUpdateEvent(OnScheduledEventUpdateABC):
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        events: EventService,
    ):
        OnScheduledEventUpdateABC.__init__(self)

        self._logger = logger
        self._bot = bot
        self._events = events

    async def on_scheduled_event_update(self, before: discord.ScheduledEvent, after: discord.ScheduledEvent):
        self._logger.debug(__name__, f"Module {type(self)} started")

        # save started event
        if before.status != after.status and after.status == EventStatus.active:
            self._events.add_event(ActiveEvent(after))
        # delete stopped event
        if before.status != after.status and (
            after.status.value == EventStatus.cancelled.value or after.status.value == EventStatus.completed.value
        ):
            event = self._events.get_active_event(after)
            if event is None:
                return
            self._events.remove_event(event)

        self._logger.debug(__name__, f"Module {type(self)} stopped")
